﻿-- PgCon14 Working with Network Data
-- James Hanson
-- Exploring our Airport, US IPGeo data and known malicious data

-- How many large airports are in the US?
SELECT 
  COUNT(*) AS num_large_airports
FROM airports
WHERE type = 'large_airport'
AND iso_country = 'US';

-- Which states are they in?
SELECT
  RIGHT(iso_region, 2) AS state,
  COUNT(*) AS num_large_airports
FROM airports
WHERE type = 'large_airport'
AND iso_country = 'US'
GROUP BY iso_region
ORDER BY num_large_airports DESC;

-- Lets look at those large US airports
SELECT *
FROM airports
WHERE type = 'large_airport'
AND iso_country = 'US';

-- Lets create a convenient view of them
CREATE OR REPLACE VIEW v_large_airports AS
SELECT *
FROM airports
WHERE type = 'large_airport'
AND iso_country = 'US';

-- DROP VIEW v_large_airpots;

-- I wonder how many of the known malicious IPs are close to large airports?
SELECT COUNT(*) AS num_blacklisted_IPs_in_us
FROM myip
  INNER JOIN us_ipgeo ui ON myip.ipaddress = ui.ip;

-- Out of how many blacklisted IPs?
SELECT to_char(count(*), '999G999') AS num_recs
FROM myip;

-- That does not seem like enough.
-- What about the known web crawlers?
SELECT COUNT(*) AS num_webcrawler_IPs_in_us
FROM myip_web
  INNER JOIN us_ipgeo ui ON myip_web.webbot_ip = ui.ip;

-- OK, we'll put those 18 records in a view
CREATE OR REPLACE VIEW v_malicious_ips_in_us AS (
  (
  SELECT 
    'webbot_ip' AS source,
    myip_web.webbot_ip as mal_ip,
    ui.city,
    ui.state,
    ui.zip_code,
    ui.location
  FROM myip_web
  INNER JOIN us_ipgeo ui ON myip_web.webbot_ip = ui.ip)
  UNION ALL
  (
  SELECT 
    'malicious_ip' AS source,
    myip.ipaddress as mal_ip,
    ui.city,
    ui.state,
    ui.zip_code,
    ui.location
  FROM myip
  INNER JOIN us_ipgeo ui ON myip.ipaddress = ui.ip)
);

-- DROP VIEW v_malicious_ips_in_us;

-- Check out our view
SELECT * 
FROM v_malicious_ips_in_us;

-- How many of the 18 bad IPs in the US are within 20 miles of a large airport?
SELECT * 
FROM v_malicious_ips_in_us 
WHERE EXISTS(
    SELECT 1 FROM v_large_airports
    WHERE ST_Distance_Sphere(v_large_airports.location, v_malicious_ips_in_us.location) <= (1609.34 * 20)  -- 1,609.34 meters / mile
)
ORDER BY mal_ip;

-- That doesn't seem right ...
-- Let's check it out.
SELECT 
  vmiis.source,
  vmiis.mal_ip,
  vmiis.city AS mal_ip_city,
  vmiis.state AS mal_ip_state,
  vmiis.zip_code AS mal_ip_zip,
  vla.ident,
  vla.name AS airport_name,
  vla.municipality AS airport_city,
  RIGHT(vla.iso_region, 2) AS airport_state,
  st_distance_sphere(vla.location, vmiis.location)/(1609.34) as distance_sm
FROM v_malicious_ips_in_us AS vmiis,
  v_large_airports AS vla
WHERE st_distance_sphere(vla.location, vmiis.location)/(1609.34) <= 20
ORDER BY 
  vmiis.state,
  vla.ident;

-- Holy cow.  15 of the 18 known malicious IPs in the US are within 20 miles of a large airport.  Who knew?
