﻿-- PgCon14 Working with Network Data
-- James Hanson
-- 070_MyIP data
-- Top 500 Hosts restructure
-- Run queries on our tophosts_range

-- look at our data
SELECT *
FROM tophosts_range;

-- explore the syntax
-- test using a single element of the array
SELECT 
  *
FROM tophosts_range
WHERE ip_range[1] @> '173.252.68.94'::inet;  -- @> = IS CONTAINED BY operator

-- use the ANY operator on the whole array
-- NOTE:  "The left-hand expression is evaluated and compared to each element of the array using the given operator, which must yield a Boolean result"
SELECT 
  *
FROM tophosts_range
WHERE '173.252.68.94'::inet <@ ANY(ip_range);

-- repeat the tests with an IP that is only in the parent_ip range
-- i.e. This IP does not appear in the Owner_IP range
SELECT 
  *
FROM tophosts_range
WHERE ip_range[1] @> '81.93.179.23'::inet; -- Remember that the [1] indicates "only look in the first array element

-- But it does show up when we query the whole array
SELECT 
  *
FROM tophosts_range
WHERE '81.93.179.23'::inet <@ ANY(ip_range);

-- Are any of the apache_log requesting IPs from a top500 site?
SELECT *
FROM apache_log
LIMIT 10;

-- This query answers the question 
--  "Are any of the big websites hitting my site?"
SELECT
  tr.website,
  tr.ww_site_rank,
  tr.hosting_country,
  tr.hosting_company,
  tr.ip_range,
  al.requesting_ip,
  al.num_occurs,
  al.action,
  al.resource,
  al.user_agent
FROM
  (SELECT 	-- Group the Apache log IPs before joining with the tophosts_range
     requesting_ip, 
     count(*) AS num_occurs,
     action, 
     resource, 
     user_agent 
    FROM apache_log 
    GROUP BY
     requesting_ip, 
     action, 
     resource, 
     user_agent 
 --   LIMIT 1000
    ) al,
  tophosts_range tr
WHERE requesting_ip <@ ANY(ip_range)
ORDER BY 
  num_occurs DESC,
  hosting_country,
  hosting_company;

-- Clean it up to remove unneeded fields
SELECT
  tr.website,
  tr.ww_site_rank,
  tr.hosting_country,
  tr.hosting_company,
  tr.ip_range,
  al.requesting_ip,
  al.num_occurs,
  al.action --,
--  al.resource,
--  al.user_agent
FROM
  (SELECT 	-- Group the Apache log IPs before joining with the tophosts_range
     requesting_ip, 
     count(*) AS num_occurs,
     action --, 
--     resource, 
--     user_agent 
    FROM apache_log 
    GROUP BY
     requesting_ip, 
     action, 
     resource, 
     user_agent 
 --   LIMIT 1000
    ) al,
  tophosts_range tr
WHERE requesting_ip <@ ANY(ip_range)
ORDER BY 
  num_occurs DESC,
  hosting_country,
  hosting_company; 