﻿-- PgCon14 Working with Network Data
-- James Hanson
-- Look at MyIP known web crawlers and malicious IPs

-- look at MyIP known web crawler data
SELECT *
FROM myip_web
LIMIT 100;

SELECT 
  webbot_ip,
  count(*)
FROM myip_web
GROUP BY webbot_ip
ORDER BY count(*) DESC;

-- See if there are any IP from the Apache Log in the MyIP known web crawler database
WITH uq_apache_ip AS (
  SELECT
    count(*) num_requests,
    max(time) most_recent,
    requesting_ip
  FROM apache_log
  GROUP BY requesting_ip)
SELECT DISTINCT  -- Ugly and cheating ... but it works
  mw.webbot_ip,
  uai.num_requests,
  uai.most_recent
FROM myip_web mw
  INNER JOIN uq_apache_ip uai ON mw.webbot_ip = uai.requesting_ip
ORDER BY num_requests DESC;

-- Are any of the known web crawler IPs in the US IPGeo data?
SELECT 
  ug.*
FROM us_ipgeo ug 
  INNER JOIN myip_web mw ON ug.ip = mw.webbot_ip
ORDER BY ip;

-- Which state are the web_crawler IPs in in?
SELECT 
  COUNT(ug.ip) num_ips,
  ug.state
FROM us_ipgeo ug 
  INNER JOIN myip_web mw ON ug.ip = mw.webbot_ip
GROUP BY ug.state
ORDER BY num_ips DESC;

-- Are any of the known web crawlers that appear in the Apache Logs also in the US?
WITH uq_apache_ip AS (
  SELECT
    count(*) num_requests,
    max(time) most_recent,
    requesting_ip
  FROM apache_log
  GROUP BY requesting_ip)
SELECT DISTINCT  -- Ugly and cheating ... but it works
  mw.webbot_ip,
  uai.num_requests,
  uai.most_recent,
  ug.city,
  ug.state,
  ug.zip_code
FROM myip_web mw
  INNER JOIN uq_apache_ip uai ON mw.webbot_ip = uai.requesting_ip
  INNER JOIN us_ipgeo ug ON ug.ip = uai.requesting_ip
ORDER BY num_requests DESC;

-- Are any of the blacklisted IPs in the Apache Access logs?
-- What is in the myIP table containing the list of blacklisted IPs
SELECT *
FROM myip
LIMIT 100;

-- Remember to pull the unique IPs from the Apache Log
WITH uq_apache_ip AS (
  SELECT
    count(*) num_requests,
    max(time) most_recent,
    requesting_ip
  FROM apache_log
  GROUP BY requesting_ip)
SELECT 
  myip.host,
  myip.countryid,
  myip.blacklistid,
  most_recent
FROM myip 
  INNER JOIN uq_apache_ip uai ON myip.ipaddress = uai.requesting_ip;

 -- What country are most of these from?
 WITH uq_apache_ip AS (
  SELECT
    count(*) num_requests,
    max(time) most_recent,
    requesting_ip
  FROM apache_log
  GROUP BY requesting_ip)
SELECT 
  myip.countryid,
  count(*) num_ips,
  max(most_recent) most_most_recent
FROM myip 
  INNER JOIN uq_apache_ip uai ON myip.ipaddress = uai.requesting_ip
GROUP BY
  myip.countryid
ORDER BY num_ips DESC;


