﻿-- PGCon14 Working with Network Data
-- James Hanson
-- Querying the National Vulneratiblities Databaes (NVD) in native XML format v1.2 with XPath

-- NOTE:  I removed the xmlns (XML Name Space) entries in the first <nvd ...> tag to make this simpler.
-- See http://stackoverflow.com/questions/17799790/using-xpath-to-extract-data-from-an-xml-column-in-postgres for details

-- Check to ensure it is a valid XML document first
SELECT
  xml_col IS DOCUMENT AS is_document
FROM nvd_12b;

SET client_encoding ='UTF-8';

-- Explore the XML tree to make sure we have the syntax correct
SELECT xpath('/nvd/*', xml_col) -- Making sure we have the correct root
FROM nvd_12b;
-- Note the error, so we have to go back to UTF-8.

SET client_encoding = 'UTF-8';

SELECT UNNEST(xpath('/nvd/entry/*', xml_col)) -- Walking down a few branches
FROM nvd_12b;

SELECT UNNEST(xpath('/nvd/entry/vuln_soft/*', xml_col)) -- Walking down a few branches
FROM nvd_12b;

SELECT UNNEST(xpath('/nvd/entry/vuln_soft/product/*', xml_col)) -- Mis-typed branch
FROM nvd_12b;

SELECT UNNEST(xpath('/nvd/entry/vuln_soft/prod/*', xml_col)) -- This is it
FROM nvd_12b;

SELECT UNNEST(xpath('/nvd/entry/vuln_soft/prod/vers[@num]', xml_col)) -- Down to a branch
FROM nvd_12b;

-- Let's look at a WHEREish clause
SELECT UNNEST(xpath('/nvd/entry/vuln_soft/prod[@vendor="oracle"]/@name', xml_col)) 
FROM nvd_12b;

-- and one for PostgreSQL
SELECT DISTINCT
  UNNEST(xpath('/nvd/entry/vuln_soft/prod[@vendor="postgresql"]/vers/@num', xml_col))::text -- note that xpath returns type XML, which cannot be sorted
FROM nvd_12b
ORDER BY 1;

-- Here are Oracle products with a High severity vulnerability 
SELECT DISTINCT     -- a product can have multiple High vulnerabilities
  UNNEST(xpath('/nvd/entry[@severity="High"]/vuln_soft/prod[@vendor="oracle"]/@name', xml_col))::text AS oracle_product
FROM nvd_12b
ORDER BY oracle_product;

-- All versions of Oracle database server that have a High serverity vulnerability
SELECT 
  UNNEST(xpath('/nvd/entry[@severity="High"]/vuln_soft/prod[@name="database_server"][@vendor="oracle"]/vers/@num', xml_col))::text AS oracle_product
FROM nvd_12b
ORDER BY oracle_product;

-- Alternating vendor and product
SELECT 
  UNNEST(xpath('/nvd/entry[@severity="High"]/vuln_soft/prod/@vendor | /nvd/entry[@severity="High"]/vuln_soft/prod/@name', xml_col)) AS high_severity
FROM nvd_12b;

-- Just playing a bit now.
-- I think this syntax will change in 9.4
-- odd elements are vendor, even elements are product
WITH pivoted_array AS (
  SELECT 
    UNNEST(xpath('/nvd/entry[@severity="High"]/vuln_soft/prod/@vendor | /nvd/entry[@severity="High"]/vuln_soft/prod/@name', xml_col)) AS high_severity
  FROM nvd_12b)
SELECT row_number() over() AS element_index,
  high_severity as element_value
FROM pivoted_array;