﻿-- PgCon14 Working with Network Data
-- James Hanson
-- Querying the tracrt and tracert_array tables

-- Look at tracert and tracert_array
SELECT *
FROM tracert
LIMIT 100;

SELECT *
FROM tracert_array
LIMIT 100;

-- Are any of these IPs in the us_ipgeo? (no)
SELECT
  tracert.*
FROM tracert 
  INNER JOIN us_ipgeo ON tracert.dest_ip = us_ipgeo.ip;

-- What about the DNS records?
SELECT *
FROM dns
LIMIT 100;

SELECT TO_CHAR(COUNT(*), '999G999G999') num_records
FROM dns;

-- Are any of these IPs in the dns records? (no)
SELECT
  tracert.*
FROM tracert 
  INNER JOIN dns ON tracert.dest_ip = dns.ip;

-- What about a DNS record for any of the hops?
SELECT
  tracert.*
FROM tracert 
  INNER JOIN dns ON tracert.to_ip = dns.ip;
-- It looks like a bunch of European IPs with DNS registrations

-- How many tracert records are there?
SELECT TO_CHAR(COUNT(*), '999G999G999') num_records
FROM tracert;

-- NOTE:  This is a join of 6.4 million DNS records and 575K tracert entries ... with no tuning on a crappy laptop!

-- Here is the syntax for pulling elements of each array
-- Looking at the most commonly used IPs to start a tracert
SELECT 
  count(*) AS num_records,
  dest_name,
  dest_ip,
  tracert_trip[1].to_ip
FROM tracert_array
WHERE dest_ip IS NOT NULL
GROUP BY 
  dest_name,
  dest_ip,
  tracert_trip[1].to_ip
HAVING count(*) >= 50
ORDER BY num_records DESC;

-- Just for fun ... here is one way to get all the from_ips or to_ips
WITH tracert_array_split AS (
  SELECT
    UNNEST(tracert_trip) AS tracert_trip
  FROM tracert_array)
SELECT 
  (tracert_trip).from_ip,
  (tracert_trip).to_ip
FROM tracert_array_split;